<?php
namespace MundiPagg\MundiPagg\Setup;

use Magento\Framework\Setup\UpgradeSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\DB\Ddl\Table;

class UpgradeSchema implements UpgradeSchemaInterface
{

    /**
     * {@inheritdoc}
     */
    public function upgrade(
        SchemaSetupInterface $setup,
        ModuleContextInterface $context
    ) {
        $setup->startSetup();
        $version = $context->getVersion();

        if (version_compare($version, "1.0.2", "<")) {
            $setup = $this->updateVersionOneZeroTwo($setup);
        }

        if (version_compare($version, "1.0.14", "<")) {
            $setup = $this->updateVersionOneZeroTwelve($setup);
        }

//Mundipagg Module Core tables
        $installSchema = new InstallSchema();

        if (version_compare($version, "1.3.0", "<")) {
            $setup = $installSchema->installWebhook($setup);
            $setup = $installSchema->installOrder($setup);
            $setup = $installSchema->installCharge($setup);
            $setup = $installSchema->installTransaction($setup);
        }

        if (version_compare($version, "1.4.0", "<")) {
            $setup = $installSchema->installConfig($setup);
            $setup = $this->fixTransactionTable($setup);
        }

        $setup->endSetup();
    }

    protected function updateVersionOneZeroTwo($setup)
    {
        $installer = $setup;
        $installer->startSetup();
 
        // Get tutorial_simplenews table
        $tableName = $installer->getTable('mundipagg_mundipagg_cards');
        // Check if the table already exists
        if ($installer->getConnection()->isTableExists($tableName) != true) {
            // Create tutorial_simplenews table
            $table = $installer->getConnection()
                ->newTable($tableName)
                ->addColumn(
                    'id',
                    Table::TYPE_INTEGER,
                    null,
                    [
                        'identity' => true,
                        'unsigned' => true,
                        'nullable' => false,
                        'primary' => true
                    ],
                    'ID'
                )
                ->addColumn(
                    'customer_id',
                    Table::TYPE_INTEGER,
                    null,
                    ['nullable' => false, 'default' => '0'],
                    'Customer Id'
                )
                ->addColumn(
                    'card_token',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false, 'default' => ''],
                    'Card Token'
                )
                ->addColumn(
                    'card_id',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false, 'default' => ''],
                    'Card Id'
                )
                ->addColumn(
                    'last_four_numbers',
                    Table::TYPE_TEXT,
                    null,
                    ['nullable' => false, 'default' => ''],
                    'Last Four Numbers'
                )
                ->addColumn(
                    'created_at',
                    Table::TYPE_DATETIME,
                    null,
                    ['nullable' => false],
                    'Created At'
                )
                ->addColumn(
                    'updated_at',
                    Table::TYPE_DATETIME,
                    null,
                    ['nullable' => false],
                    'Updated At'
                )
                ->setComment('Mundipagg Card Tokens')
                ->setOption('type', 'InnoDB')
                ->setOption('charset', 'utf8');
            $installer->getConnection()->createTable($table);
        }
 
        $installer->endSetup();

        return $setup;
    }

    protected function updateVersionOneZeroTwelve($setup)
    {
        $installer = $setup;
        $installer->startSetup();

        $connection = $installer->getConnection();

        $connection->addColumn(
            $installer->getTable('mundipagg_mundipagg_cards'),
            'brand',
            [
                'type' => Table::TYPE_TEXT,
                'length' => 255,
                'nullable' => false,
                'default' => '',
                'comment' => 'Card Brand'
            ]
        );

        $installer->endSetup();

        return $setup;
    }

    protected function fixTransactionTable($setup)
    {
        $installer = $setup;

        $connection = $installer->getConnection();

        $connection->modifyColumn(
            $installer->getTable('mundipagg_module_core_transaction'),
            'acquirer_tid',
            [
                'type' => Table::TYPE_TEXT,
                'length' => 300,
            ]
        )
        ->modifyColumn(
            $installer->getTable('mundipagg_module_core_transaction'),
            'acquirer_nsu',
            [
                'type' => Table::TYPE_TEXT,
                'length' => 300,
            ]
        )
        ->modifyColumn(
            $installer->getTable('mundipagg_module_core_transaction'),
            'acquirer_auth_code',
            [
                'type' => Table::TYPE_TEXT,
                'length' => 300,
            ]
        );

        return $setup;
    }
}